#ifndef _LIST_H
#define _LIST_H 1


/* Include a LISTABLE declaration in structures that you want to keep in 
   linked lists, supplying the type of the structure itself. It expands to
   a declaration for a *next and *prev pointer, using a similar style as the
   SLOTTABLE declaration generates a slot in use flag for slot arrays. You
   can specify an optional prefix, to declare membership of multiple lists. */

#define LISTABLE(type)		type *next, *prev
#define LISTABLEP(type, pfx...)	type *pfx##next, *pfx##prev


/* List management macros. The head and tail arguments must be pointers to
   the actual head and tail variables, which must be pointers of the same 
   type as obj. Objects are required to have 'next' and 'prev' pointers of
   the same type. You can specify an optional prefix for the their names. */

#define LIST_ADDTAIL(head, tail, obj)					\
    do {								\
	(obj)->prev = *(tail);						\
	(obj)->next = 0;						\
	if (*(tail)) (*(tail))->next = (obj);				\
	else *(head) = (obj);						\
	*(tail) = (obj);						\
    } while(0)

#define LIST_ADDTAILP(head, tail, obj, pfx...)				\
    do {								\
	(obj)->pfx##prev = *(tail);					\
	(obj)->pfx##next = 0;						\
	if (*(tail)) (*(tail))->pfx##next = (obj);			\
	else *(head) = (obj);						\
	*(tail) = (obj);						\
    } while(0)

#define LIST_ADDLISTTAILP(head, tail, firstobj, lastobj, pfx...)	\
    do {								\
	(firstobj)->pfx##prev = *(tail);				\
	(lastobj)->pfx##next = 0;					\
	if (*(tail)) (*(tail))->pfx##next = (firstobj);			\
	else *(head) = (firstobj);					\
	*(tail) = (lastobj);						\
    } while(0)

#define LIST_ADDHEAD(head, tail, obj)					\
    do {								\
	(obj)->next = *(head);						\
	(obj)->prev = 0;						\
	if (*(head)) (*(head))->prev = (obj);				\
	else *(tail) = (obj);						\
	*(head) = (obj);						\
    } while(0)

#define LIST_ADDHEADP(head, tail, obj, pfx...)				\
    do {								\
	(obj)->pfx##next = *(head);					\
	(obj)->prev = 0;						\
	if (*(head)) (*(head))->pfx##prev = (obj);			\
	else *(tail) = (obj);						\
	*(head) = (obj);						\
    } while(0)

#define LIST_ADDLISTHEADP(head, tail, firstobj, lastobj, pfx...)	\
    do {								\
	(lastobj)->pfx##next = *(head);					\
	(firstobj)->prev = 0;						\
	if (*(head)) (*(head))->pfx##prev = (lastobj);			\
	else *(tail) = (last);						\
	*(head) = (firstobj);						\
    } while(0)

#define LIST_DEL(head, tail, obj)					\
    do {								\
	if ((obj)->next) (obj)->next->prev = (obj)->prev;		\
	if ((obj)->prev) (obj)->prev->next = (obj)->next;		\
	if ((obj) == *(head)) *(head) = (obj)->next;			\
	if ((obj) == *(tail)) *(tail) = (obj)->prev;			\
    } while(0)

#define LIST_DELP(head, tail, obj, pfx...)				\
    do {								\
	if ((obj)->pfx##next) (obj)->pfx##next->pfx##prev = (obj)->pfx##prev; \
	if ((obj)->pfx##prev) (obj)->pfx##prev->pfx##next = (obj)->pfx##next; \
	if ((obj) == *(head)) *(head) = (obj)->pfx##next;		\
	if ((obj) == *(tail)) *(tail) = (obj)->pfx##prev;		\
    } while(0)


#endif


/*
 * vim:softtabstop=4:sw=4
 */

