
/* 
   Remember when I wrote this is a library for new code, not a porting aid? 

   This is an exception that confirms the rule: an abstraction above select/
   poll/WaitForMultipleObjects. It provides an abstract file handle as well
   (of course, it's unforgivable that Windows doesn't have proper file 
   descriptors at the kernel level) and an abstract time/date value with
   microsecond precision or better.
*/


#ifndef HANDLESET_H
#define HANDLESET_H 1


/* Platform-specific definitions for abstract types 
   FD_T, TIMER_T and HANDLESET_T */

#ifndef WINDOWS 		/* Any platform with a POSIX-like interface */

#include <sys/time.h>
#include <time.h>

typedef int 		FD_T;
typedef struct timeval	TIMER_T;

#ifndef timerisset		/* define the standard BSD macros 
				   if we don't have them */

#define       timerisset(tvp) \
       ((tvp)->tv_sec || (tvp)->tv_usec)

#define       timercmp(tvp, uvp, cmp) \
       ((tvp)->tv_sec cmp (uvp)->tv_sec || \
        ((tvp)->tv_sec == (uvp)->tv_sec && \
         (tvp)->tv_usec cmp (uvp)->tv_usec))

#define       timerclear(tvp) \
       ((tvp)->tv_sec = (tvp)->tv_usec = 0)

#endif

#ifndef USE_POLL 		/* Unix w/ select */

#include <sys/types.h>
#include <sys/select.h>
#include <unistd.h>

typedef struct handleset
{
	int highestfd;
	fd_set rfds;
	fd_set wfds;
	TIMER_T firsttimer;

} HANDLESET_T;

#else				/* Unix w/ poll */

#include <sys/poll.h>

#ifndef MAX_POLL_SET
#define MAX_POLL_SET 	1024
#endif

typedef struct handleset
{
	int pollfdcount;
	struct pollfd pollfds[ MAX_POLL_SET ];
	TIMER_T firsttimer;

} HANDLESET_T;

#endif

#else				/* Windows */

#define WIN32_LEAN_AND_MEAN	/* uhuh */
#include <windows.h>

typedef HANDLE		FD_T;
typedef FILETIME	TIMER_T;

#ifndef MAX_WFMO_SET 
#define MAX_WFMO_SET 	256
#endif

typedef struct handleset
{
	int handlecount;
	HANDLE handles[ MAX_WFMO_SET ];
	TIMER_T firsttimer;

} HANDLESET_T;

#endif


/* Prototypes */

void timer_now( TIMER_T *dest );
void timer_sub( TIMER_T *dest, TIMER_T *left, TIMER_T *right );	
void timer_addsec( TIMER_T *dest, TIMER_T *src, long seconds );
void timer_addmicro( TIMER_T *dest, TIMER_T *src, long microseconds );

int  handleset_init( HANDLESET_T *hs );

void handleset_addtimer( HANDLESET_T *hs, TIMER_T *timer );

void handleset_addfd( HANDLESET_T *hs, FD_T handle, int for_write );
int  handleset_hasfd( HANDLESET_T *hs, FD_T handle, int for_write );
void handleset_remfd( HANDLESET_T *hs, FD_T handle, int for_write );

void handleset_addhs( HANDLESET_T *hs, HANDLESET_T *src_hs );
void handleset_remhs( HANDLESET_T *hs, HANDLESET_T *src_hs );

void handleset_clear( HANDLESET_T *hs );

int  handleset_wait( HANDLESET_T *io_hs, TIMER_T *io_curtime );

void handleset_done( HANDLESET_T *hs );


#endif /* HANDLESET_H */

