
#include <evblib/str/str.h>
#include <evblib/sysdefs/systypes.h>

#ifndef WINDOWS
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#else
#include <winsock2.h>
#define SHUT_RDWR SD_BOTH
#endif


/* CONNECTION-ORIENTED SERVERS */

/* returns listening socket on success, -errno on error. */

int listensock_init( int addrfamily, char *addrp, size_t addrl,
		   int socktype, int protocol, int port );

/* idem, specifically for ipv4 sockets; address given in host order (eg.
   192.168.5.1 is 0xc0a80501). */

int listensock_init_ipv4( uint32_t ipv4addr, 
			int socktype, int protocol, int port );

/* returns connected socket on success, -errno on error; can be called on
   listening socket and blocks for a connection in that case; can also be
   called after selecting on the listening socket. */

int accept_ipv4( int listeningfd, uint32_t *fromipv4addr, int *fromport );


/* CLIENTS AND CONNECTIONLESS ENDPOINTS */

/* returns new unbound socket on success, -errno on error. */

int socket_init( int addrfamily, int socktype, int protocol );

/* returns socket with local endpoint bound to a local address on success, 
   -errno on error. */

int boundsocket_init( int addrfamily, char *addrp, size_t addrl,
		      int socktype, int protocol, int port );

/* idem, specifically for ipv4 sockets; address given in host order */

int boundsocket_init_ipv4( uint32_t ipv4addr, 
			   int socktype, int protocol, int port );


/* CONNECTION-ORIENTED CLIENTS */

/* connects remote endpoint of a socket to a remote address; can be used on 
   bound sockets, which may be created with boundsocket_init(), or unbound 
   sockets as created with the normal socket() call. Blocks until the 
   connection is established, unless the socket was set to nonblocking mode 
   first, using eg. setblockingmode(). Returns 0 on success, -errno on error. */

int socket_connect( int sockfd, int addrfamily, char *addrp, size_t addrl,
					int port );

/* idem, specifically for ipv4 endpoints */

int socket_connect_ipv4( int sockfd, uint32_t ipv4addr, int port );


/* MISCELLANEOUS */

/* sets a socket (any state; newly created, bound, listening or connected)
   to blocking or nonblocking mode; returns 0 on success, -errno on error */

int setblockingmode( int fd, int blocking );

/* returns ascii representation of given ipv4 address in newly allocated 
   string; free with FREESTR. */

STR_T ipv4tostr( uint32_t ipv4addr );

/* returns ipv4 address from dotted-decimal string; if ret_error is nonzero,
   writes an integer at the given address indicating a valid syntax of the 
   input string, 0 if valid, <0 otherwise. */

uint32_t strtoipv4( char *addrstr, size_t addrstrl, 
		    int *ret_error );

/* returns primary ipv4 address of machine, using platform specific method */

uint32_t ownipv4addr();

/* tries to interpret hostname as decimal address, performs blocking lookup of 
   name if that fails; returns first ipv4 address given by lookup. If 
   ret_error is nonzero, writes an integer at the given address indicating
   success, 0 if the translation, resp. lookup was successful; 
   -errno otherwise. */

uint32_t getipv4addrbyname( char *hostname, size_t hostnamel, 
			    int *ret_error );

/* performs blocking lookup of hostname based on ipv4 address; if success,
   returns first address, else returns dotted decimal string as returned
   by ipv4tostr. Returns newly allocated string; free with FREESTR. */

STR_T getnamebyipv4addr( uint32_t ipv4addr );


/*
 * vim:ts=4:sw=4
 */

